/************************************************************
	FILE:  com1.c

************************************************************/

#include "defs.h"
#include "globals.h"
#include <math.h>

extern dyn_chand();
extern dyn_pend1();
extern dyn_pend2();
extern dyn_pend4();
extern dyn_luxo();
extern mon_chand();
extern mon_pend1();
extern mon_pend2();
extern mon_pend4();
extern mon_luxo();

struct dynlist {
	STR name;
	PFI fn;
} Dyn_list[] = {
	"chand",dyn_chand,
	"pend1",dyn_pend1,
	"pend2",dyn_pend2,
	"pend4",dyn_pend4,
	"luxo",dyn_luxo,
	"mon_chand",mon_chand,
	"mon_pend1",mon_pend1,
	"mon_pend2",mon_pend2,
	"mon_pend4",mon_pend4,
	"mon_luxo",mon_luxo,
	"",NULL
};

/************************************************************
	PROC:	dyn()
	DOES:	choose dynamics procedure
************************************************************/

dyn(nargs,args)
int nargs;
STR args[MAX_ARGS];
{
	extern PFI dyn_find();

	if (nargs==1) {			/* list available procedures */
		pr_dyn();
		return;
	}
	if (nargs!=2) {
		cerr(nargs,args,"Bad number of arguments");
		return;
	}
	Dyn = dyn_find(args[1]);		/* find dynamics procedure */
}


/************************************************************
	PROC:	mon()
	DOES:	choose monitor procedure
************************************************************/

mon(nargs,args)
int nargs;
STR args[MAX_ARGS];
{
	extern PFI dyn_find();

	if (nargs==1) {			/* list available procedures */
		pr_dyn();
		return;
	}
	if (nargs!=3) {
		cerr(nargs,args,"Bad number of arguments");
		return;
	}
	Mon = dyn_find(args[1]);		/* find dynamics procedure */
	Nmon = atoi(args[2]);
}

/************************************************************
	PROC:	pr_dyn()
	DOES:	prints the names of available dynamics procedures
************************************************************/

pr_dyn()
{
	int n = 0;

	while (Dyn_list[n].name[0]) {
		printf("\t%s\n",Dyn_list[n].name);
		n++;
	}
}

/************************************************************
	PROC:	dyn_find()
	DOES:	looksup a dynamics procedure by name
************************************************************/

PFI dyn_find(fn_name)
STR fn_name;
{
	int n=0;
	
	while (Dyn_list[n].name[0]) {
		if (!strcmp(Dyn_list[n].name,fn_name))
			return(Dyn_list[n].fn);
		n++;
	}
	printf("Dynamics procedure '%s' not found.\n",fn_name);
	return((PFI) NULL);
}

/************************************************************
	PROC:	state()
	DOES:	sets or prints the current system state 
************************************************************/

state(nargs,args)
int nargs;
STR args[];
{
	int argc;
	STR argv[LVECSIZE];
	extern float atof();
	int n;

	if (nargs==1) {
		prstate();
		return;
	}
	if (nargs!=2) {
		cerr(nargs,args,"Bad number of arguments");
		return;
	}
	parse(args[1],",",&argc,argv);		/* parse vector elements */
	for (n=0; n<argc; n++)
		State[n] = atof(argv[n]);
	prstate();
}

/************************************************************
	PROC:	prstate()
	DOES:	prints the current system state
************************************************************/

prstate()
{
	int n;

	for (n=0; n<STate_size; n++) {
		printf("%6.3f ",State[n]);
	}
	printf("\n");
}

/************************************************************
	PROC:	sim()
	DOES:	simulates the system motion
************************************************************/

sim(nargs,args)
int nargs;
STR args[];
{
	float end_time;
	float disp_time = -0.0001;
	extern float atof();
	LFVEC torq;		/* torques to be applied */
	LFVEC fext;		/* forces applied at monitor points */
	LFVEC ac;		/* system acceleration */
	int n;
	float t;

	Fp_disp = fopen(DIspfile,"w");
	end_time = atof(args[1]);
	for (n=0; n<LVECSIZE; n++)			/* set all external forces to zero */
		fext[n] = 0.0;
	for (Time=0.0; Time<end_time; Time+=DTsim) {	/* do simulation */
		for (n=0; n<7; n++)
			torq[n] = -State[4+n*2+1]*KDamp[n];	/* friction dampening */
		control(torq);						/* calculate control */
		if (Mon) ground_force(fext);		/* ground collisions */
		(*Dyn)(State,torq,fext,ac);			/* calc accelerations */
		integ(State,ac);					/* integrate */
		if (Time>disp_time) {					/* time to display? */
			display();						/* display */
			disp_time += DTdisp;
		}
	}
	fclose(Fp_disp);
}

/************************************************************
	PROC:	control(torq)
	DOES:	calculates some primitive controlling torques
************************************************************/

control(torq)
LFVEC torq;
{
	float kp,kd;
	extern dyn_luxo();

	if (Dyn!=dyn_luxo) return;		/* for Luxo only */
	kp = 1.0;
	kd = 0.1;

	set_torq(torq,1,-1.0 + sin(3.0*Time),kp,kd);
	set_torq(torq,2,0.7 - sin(3.0*Time),kp,kd);
}

set_torq(torq,joint,dposn,kp,kd)
LFVEC torq;
int joint;
float dposn,kp,kd;
{
	float joint_posn;
	float joint_vel;

	joint_posn = State[4 + joint*2];
	joint_vel = State[4 + joint*2 + 1];
	torq[joint] = kp*(dposn - joint_posn) - kd*joint_vel;
}

/************************************************************
	PROC:	ground_force(fext)
	DOES:	determines external forces on links
************************************************************/

#define KSPRING 700
#define KDAMP 40

ground_force(fext)
LFVEC fext;
{
	float mstate[50];
	int pt;
	int i1,i2,i3;					/* indices */
	float gnd;
	float fx,fy;
	float sinx,cosx,temp;
	float fxn,fyn;					/* ground normal forces */

	(*Mon)(State,mstate);
	for (pt=0; pt<Nmon; pt++) {
		i1 = pt*4;					/* index for mstate */
		i2 = pt*ENTRY_SIZE;			/* index for Foot_state */
		i3 = pt*2;					/* index for external forces */
		gnd = 0.0;					/* ground height */
		if (mstate[i1+2]>gnd) {		/* if not in contact */
			Foot_state[i2+CONTACT] = 0.0;
			fx = fy = 0.0;
		} else {					/* if in contact */
			if (Foot_state[i2+CONTACT]==0.0) {		/* first contact? */
				Foot_state[i2+CONTACT] = 1.0;		/* mark as in contact */
				Foot_state[i2+XPOSN] = mstate[i1];	/* save x coord */
				Foot_state[i2+YPOSN] = gnd;			/* save y coord */
			}
			fx = (Foot_state[i2+XPOSN]-mstate[i1])*KSPRING -
						mstate[i1+1]*KDAMP;
			fy = (Foot_state[i2+YPOSN]-mstate[i1+2])*KSPRING -
						mstate[i1+3]*KDAMP;
			if (fy<0.0) {
				fy = 0.0;
				fx = 0.0;
			}
		}
		fext[i3] = fx;
		fext[i3+1] = fy;
	}
}

/************************************************************
	PROC:	display()
	DOES:	records the current state for later display
************************************************************/

display()
{
	int n;

	fprintf(Fp_disp,"show ");
	for (n=0; n<STate_size; n+=2)
		fprintf(Fp_disp,"%6.3f ",State[n]);
	fprintf(Fp_disp,"\n");
}

/************************************************************
	PROC:	integ()
	DOES:	integrates the system state
************************************************************/

integ(state,ac)
float state[];
float ac[];
{
	int n;

	for (n=0; n<STate_size; n+=2) {
			/* update position */
		state[n] += state[n+1]*DTsim + 0.5*ac[n/2]*DTsim*DTsim;
			/* update velocity */
		state[n+1] += ac[n/2]*DTsim;
	}
}
