/************************************************************
	FILE:		com.c
	CONTAINS:	simple commands
	Michiel van de Panne, University of Toronto
************************************************************/

#include "defs.h"
#include "globals.h"

/************************************************************
	PROC:	quit(argc,argv)
	DOES:	lists dynamics equations available for use
************************************************************/

void quit(int argc, STR argv[])
{
	exit(0);
}

/************************************************************
	PROC:	help
	DOES:	prints appropriate section from help file
************************************************************/

void help(int argc, STR argv[])
{
	FILE *help_fp;			/* pointer to help file */
	STR	line;				/* line of help file */
	BOOL print=FALSE;		/* printing flag */
	BOOL found=FALSE;		/* found any matches? */
	BOOL mismatch=FALSE;	/* mismatch in comparison */
	int ch,n;				/* counters */
	int len;				/* line length */
	int wordlen;			/* length of search word */

	if (argc==1) strcpy(argv[1],"help");
	help_fp = fopen(HElpfile,"r");
	if (!help_fp) {
		fprintf(stderr,"help file not found\n");
		fprintf(stderr,"Expecting it to be: %s\n",HElpfile);
		return;
	}
	wordlen = strlen(argv[1]);
	while (!feof(help_fp)) {				/* for each line */
		fgets(line,STRLEN,help_fp);
		if (line[0] == ':') {				/* a topic line? */
			print = FALSE;				/* turn off printing */
			len = strlen(line);
			ch = 0;
			while (1) {
				while (ch<len && line[ch]!=':')	/* look for ':' */
					ch++;
				if (ch>=len) break;		/* no more entries ? */
				ch++;
				mismatch = FALSE;
				for (n=0; n<wordlen; n++)	/* check for match */
					if (argv[1][n] != line[ch+n]) {
						mismatch = TRUE;
						ch += n;
						break;
					}
				if (!mismatch) {		/* found a match */
					print = found = TRUE;
				}
			}
		} else if (print)
			printf("%s",line);
	}
	if (!found) {
		printf("Help for %s not found.\n",argv[1]);
		printf("Use 'help sum' for a summary of commands\n");
	}
}

/************************************************************
	PROC:	pipe_in
	DOES:	used to source input from files
************************************************************/

void pipe_in(int argc, STR argv[])
{
	FILE *fp = NULL;
	int mode;
	STR fname;

	strcpy(fname,argv[1]);
	fp = fopen(fname,"r");
	if (!fp) {
		cerr(argc,argv,"Could not find file");
		return;
	}
	mode = IMode;
	IMode = FALSE;	/* go out of interactive mode */
	interp(fp);	/* read in file & interpret */
	IMode = mode;
}

/************************************************************
	PROC:	echo(argc,argv)
	DOES:	prints its arguments
************************************************************/

void echo(int argc, STR argv[])
{
	int n;

	for (n=1; n<argc; n++)
		printf("%s ",argv[n]);
	printf("\n");
}

/************************************************************
	PROC:	cat(argc,argv)
	DOES:	lists the contents of a file
************************************************************/

void cat(int argc, STR argv[])
{
	FILE *fptr;
	char line[255];
	int n;

	for (n=1; n<argc; n++) {
		fptr = fopen(argv[n],"r");
		if (!fptr) {
			printf("Unable to open file '%s'\n",argv[n]);
			continue;
		}
		while (!feof(fptr)) {
			fgets(line,255,fptr);
			fputs(line,stdout);
		}
		fclose(fptr);
	}
}

/************************************************************
	PROC:	test
	DOES:	used for testing new procedures
************************************************************/

void test(int argc, STR argv[])
{
	if (DEbug)
	printf("In procedure test()\n");
}

