/************************************************************
	FILE: cm1.cpp
	Michiel van de Panne, University of Toronto
************************************************************/

#include "defs.h"
#include "globals.h"

/************************************************************
	PROC:	compile()
	DOES:	compiles equations of motion
************************************************************/

void compile(int nargs, STR args[])
{
	int linklist[MAX_LINKS];
	STR fname;
	FILE *file;

	linklist[0]=0;
	link_children();				/* set up parent/child ptrs in LINK */
	findroot();						/* find root link, set acc for base */
	stats();						/* gather some statistics */
	gen_vars();						/* generate names of unknowns */
	if (DEbug)
		prVars();

	rcalclist(Rlink,linklist);		/* calculate linklist for every link */
	calc_accel(Rlink);				/* calculate acceleration equations */
	calc_force(Rlink);				/* calculate forces between links */
	gen_t_eqn();					/* generate equations for torque */
	if (DEbug) {
		pr_Eqns(&Ptr_eqns,"Ptr_eqns");
	}
	reduce();						/* collects constants */

	find_solve_eqns();				/* find solve equations */
	flat_solve();					/* flatten required equations */

	printf("%d const eqns\n",Const_eqns.num_eqns);
	printf("%d ptr eqns\n",Ptr_eqns.num_eqns);
	if (DEbug) {
		pr_Eqns(&Const_eqns,"Const_eqns");
		pr_Eqns(&Ptr_eqns,"Ptr_eqns");
	}
	gem();							/* place equations in matrix */
	if (DEbug)	prVars(); 
	if (DEbug)	prLabels();
	if (DEbug)	pr_eqn_mat();
	if (FIlename[0])
		sprintf(fname,"%s/dyn_%s.c",DIrname,FIlename);		/* file for output */
	else
		sprintf(fname,"%s/dyn_%s.c",DIrname,PRocname);		/* file for output */
	file = fopen(fname,"w");
	gen_file(file);
	mon_mark(Rlink);
	gen_mon_eqns(file);
	fclose(file);
}

/************************************************************
	PROC:	link
	DOES:	defines a link as part of the structure
************************************************************/

void create_link(int nargs, STR args[])
{
	LINK *temp;

	temp = Link;
	Link = (LINK *) mmalloc(sizeof(LINK));
	sscanf(args[1],"%d",&(Link->num));		/* number */
	sscanf(args[2],"%d",&(Link->parent));	/* parent number */
	sscanf(args[3],"%f",&(Link->parx));		/* parent x */
	sscanf(args[4],"%f",&(Link->pary));		/* parent y */
	sscanf(args[5],"%f",&(Link->mass));		/* mass */
	sscanf(args[6],"%f",&(Link->iner));		/* inertia */
	sscanf(args[7],"%f",&(Link->cmx));		/* center of mass, x */
	sscanf(args[8],"%f",&(Link->cmy));		/* center of mass, y */
	
	Link->child = (CHILD *) NULL;			/* no children yet */
	Link->next = temp;
	Link->fix = FALSE;
	Link->mon = (MON *) NULL;
}

/************************************************************
	PROC:	fix
	DOES:	fixes the position of the endpoint of a given link
************************************************************/

void fix(int nargs, STR args[])
{
	LINK *link;
	int link_num;

	sscanf(args[1],"%d",&link_num);
	link = llookup(link_num);
	if (!link) {
		cerr(nargs,args,"Could not find link");
		return;
	}
	link->fix = TRUE;
	sscanf(args[2],"%f",&link->pfx);
	sscanf(args[3],"%f",&link->pfy);
}

/************************************************************
	PROC:	monitor
	DOES:	specifies a point where external forces are to be applied
************************************************************/

void monitor(int nargs, STR args[])
{
	LINK *link;
	int link_num;
	MON *new_mon;

	sscanf(args[1],"%d",&link_num);
	link = llookup(link_num);
	if (!link) {
		cerr(nargs,args,"Could not find link");
		return;
	}
	new_mon = (MON *) mmalloc(sizeof(MON));
	new_mon->next = link->mon;
	link->mon = new_mon;
	sscanf(args[3],"%f",&new_mon->x);
	sscanf(args[4],"%f",&new_mon->y);
	sscanf(args[2],"%d",&new_mon->num);
}

