/************************************************************
	FILE: com2.c

************************************************************/

#include "defs.h"
#include "globals.h"

/************************************************************
	PROC:	link_children()
	DOES:	sets up pointers to the children of a given link,
		because in the specification, only parents are given
************************************************************/

void link_children()
{
	LINK *temp,*link;
	CHILD *temp_child;

	link = Link;
	while (link) {						/* for each link */
		if (link->parent==0) {				/* skip root link */
			link = link->next;
			continue;
		}
		temp = Link;					/* search for the parent node */
		while (temp) {
			if (temp->num == link->parent) break;
			temp = temp->next;
		}
		if (!temp) {
			printf("Parent %d not found\n",link->parent);
			return;
		}
		temp_child = temp->child;		/* add the link as a child of parent */
		temp->child = (CHILD *) mmalloc(sizeof(CHILD));
		temp->child->link = link;
		temp->child->next = temp_child;
		link = link->next;
	}
}

/************************************************************
	PROC:	pr_children
	DOES:	prints children of links
************************************************************/

void pr_children()
{
	LINK *link;
	CHILD *child;

	link = Link;
	while (link) {
		child = link->child;
		printf("CHILD %d: ",link->num);
		while (child) {
			printf("%d ",child->link->num);
			child = child->next;
		}
		printf("\n");
		link = link->next;
	}
}

/************************************************************
	PROC:	paddstr
	DOES:	Adds a product term to the end of the linked list
************************************************************/

void paddstr( TERM *term, char *str)
{
	P *p,*p1;

	p = (P *) mmalloc(sizeof(P));			/* create new space */
	p1 = term->p;
	if (p1) {
		while (p1->next) p1=p1->next;		/* traverse to end of list */
		p1->next = p;
	} else term->p = p;						/* or start list */
	p->next = (P *) NULL;					/* mark end of list */
	strcpy(p->txt,str);
}

/************************************************************
	PROC:	paddsin
	DOES:	Adds a product term - sin
************************************************************/

void paddsin(TERM *term, char *str)
{
	P *p,*p1;

	p = (P *) mmalloc(sizeof(P));
	p1 = term->p;
	while (p1->next) p1=p1->next;
	p1->next = p;
	p->next = (P *) NULL;
	strcpy(p->txt,"sin");
	strcat(p->txt,str);
}

/************************************************************
	PROC:	paddcos
	DOES:	Adds a product term - cos
************************************************************/

void paddcos(TERM *term, char *str)
{
	P *p,*p1;

	p = (P *) mmalloc(sizeof(P));
	p1 = term->p;
	while (p1->next) p1=p1->next;
	p1->next = p;
	p->next = (P *) NULL;
	strcpy(p->txt,"cos");
	strcat(p->txt,str);
}

/************************************************************
	PROC:	eqncpy(from)
	DOES:	creates a copy of an equation
************************************************************/

TERM *eqncpy(TERM *from)
{
	TERM *temp,*newto,*to;
	P *pfrom,*pnewto,*ptemp;

	to = (TERM *) NULL;
	while (from) {
		temp = to;
		if (temp) while (temp->next) temp=temp->next;
		newto = (TERM *) mmalloc(sizeof(TERM));
		newto->neg = from->neg;
		newto->next = (TERM *) NULL;
		if (temp) temp->next = newto;
		else to = newto;
		pfrom = from->p;
		while (pfrom) {
			ptemp = newto->p;
			if (ptemp) while (ptemp->next) ptemp=ptemp->next;
			pnewto = (P *) mmalloc(sizeof(P));
			pnewto->next = (P *) NULL;
			strcpy(pnewto->txt,pfrom->txt);
			if (ptemp) ptemp->next = pnewto;
			else newto->p = pnewto;
			pfrom = pfrom->next;
		}
		from = from->next;
	}
	return(to);
}

LINK *llookup(int num)
{
	LINK *link;

	link = Link;
	while (link) {
		if (link->num == num) return(link);
		link = link->next;
	}
	printf("Unable to find link number %d\n",num);
	return(NULL);
}

/************************************************************
	PROC:	buildlist
	DOES:	builds a sum of terms list
************************************************************/

void buildlist(LINK *link)
{
	STR temp;
	MON *mon;

	new_const_eqn("th",link->num);			/* const eqn for th */
	new_const_term(POS);
	sprintf(temp,"st[%d]",link->num*2+2);
	new_const_var(temp);
	if (link!=Rlink) {
		new_const_term(POS);
		sprintf(temp,"th%d",link->parent);
		new_const_var(temp);
	}

	new_const_eqn("dth",link->num);			/* const eqn for dth */
	new_const_term(POS);
	sprintf(temp,"st[%d]",link->num*2+3);
	new_const_var(temp);
	if (link!=Rlink) {
		new_const_term(POS);
		sprintf(temp,"dth%d",link->parent);
		new_const_var(temp);
	}

	new_const_eqn("cm_angle",link->num);	/* const eqn for cm_angle */
	new_const_term(POS);
	sprintf(temp,"th%d",link->num);
	new_const_var(temp);
	new_const_term(POS);
	sprintf(temp,"A_CM%d",link->num);
	new_const_var(temp);

	new_const_eqn("sin_cm",link->num);		/* const eqn for sin_cm */
	new_const_term(POS);
	sprintf(temp,"sin(cm_angle%d)",link->num);
	new_const_var(temp);

	new_const_eqn("cos_cm",link->num);		/* const eqn for cos_cm */
	new_const_term(POS);
	sprintf(temp,"cos(cm_angle%d)",link->num);
	new_const_var(temp);

	if (link->parent) {
		new_const_eqn("at_angle",link->num);	/* const eqn for at_angle */
		new_const_term(POS);
		sprintf(temp,"th%d",link->parent);
		new_const_var(temp);
		new_const_term(POS);
		sprintf(temp,"A_AT%d",link->num);
		new_const_var(temp);
		
		new_const_eqn("sin_at",link->num);		/* const eqn for sin_at */
		new_const_term(POS);
		sprintf(temp,"sin(at_angle%d)",link->num);
		new_const_var(temp);
		
		new_const_eqn("cos_at",link->num);		/* const eqn for cos_at */
		new_const_term(POS);
		sprintf(temp,"cos(at_angle%d)",link->num);
		new_const_var(temp);
	}

	if (link->fix) {
		new_const_eqn("f_angle",link->num);		/* const eqn for f_angle */
		new_const_term(POS);
		sprintf(temp,"th%d",link->num);
		new_const_var(temp);
		new_const_term(POS);
		sprintf(temp,"A_F%d",link->num);
		new_const_var(temp);
		
		new_const_eqn("sin_f",link->num);		/* const eqn for sin_f */
		new_const_term(POS);
		sprintf(temp,"sin(f_angle%d)",link->num);
		new_const_var(temp);
		
		new_const_eqn("cos_f",link->num);		/* const eqn for cos_f */
		new_const_term(POS);
		sprintf(temp,"cos(f_angle%d)",link->num);
		new_const_var(temp);
	}
	for (mon=link->mon; mon; mon=mon->next) {
		new_const_eqn("m_angle",mon->num);
		new_const_term(POS);
		sprintf(temp,"th%d",link->num);
		new_const_var(temp);
		new_const_term(POS);
		sprintf(temp,"A_M%d",mon->num);
		new_const_var(temp);

		new_const_eqn("sin_m",mon->num);
		new_const_term(POS);
		sprintf(temp,"sin(m_angle%d)",mon->num);
		new_const_var(temp);

		new_const_eqn("cos_m",mon->num);
		new_const_term(POS);
		sprintf(temp,"cos(m_angle%d)",mon->num);
		new_const_var(temp);
	}
}

//////////////////////////////////////////////////////
// PROC: mmalloc()
// DOES: malloc, with an error check
//////////////////////////////////////////////////////

char *mmalloc(int size)
{
	char *ret;

	ret = (char *) malloc((unsigned) size);
	if (!ret) {
		fprintf(stderr,"Bad malloc\n");
		exit(-1);
	}
	return(ret);
}

/************************************************************
	PROC:	findroot()
	DOES:	finds the link with no parent
************************************************************/

void findroot()
{
	TERM *ay = (TERM *) NULL;
	TERM *ax = (TERM *) NULL;
	extern TERM *newterm();
	LINK *link;

	link = Link;
	while (link) {
		if (link->parent == 0) {
			Rlink = link;			/* setup pointer to root link */
			Rt = link->num;			/* save number of root link */
			return;
		}
		link = link->next;
	}
	fprintf(stderr,"Root not found...\n");
	exit(-1);
}

/************************************************************
	PROC:	gen_t_eqn
	DOES:	generates sum of T=0 equation
			Moments are summed around the origin, ie the inboard joint
************************************************************/

void gen_t_eqn()
{
	LINK *link;

	for (link=Link; link; link=link->next) {
		new_ptr_eqn("ptr_teqn",link->num);
		add_t_cm(link);
		add_t_parent(link);
		add_t_child(link);
		if (link->fix) add_t_fix(link);
		add_t_ext(link);
		add_t_iw(link);
	}
}

/************************************************************
	PROC:	add_t_cm(link)
	DOES:	adds torque due to acceration of center of mass
************************************************************/

void add_t_cm(LINK *link)
{
	STR acxn,acyn,massn,len;
	STR temp;

		/* ready terms */
	sprintf(massn,"MASS%d",link->num);
	sprintf(acxn,"acx%d",link->num);
	sprintf(acyn,"acy%d",link->num);
	sprintf(len,"LENCM%d",link->num);

	sprintf(acxn,"ptr_acx%d",link->num);
	sprintf(acyn,"ptr_acy%d",link->num);
	new_ptr_term(POS);			/* gravity */
	new_ptr_var(massn);
	new_ptr_var(GRAVITY);
	new_ptr_var(len);
	sprintf(temp,"cos_cm%d",link->num);
	new_ptr_var(temp);
	new_ptr_term(POS);			/* mass*ax */
	new_ptr_var(massn);
	new_ptr_var(acxn);
	new_ptr_var(len);
	sprintf(temp,"sin_cm%d",link->num);
	new_ptr_var(temp);
	new_ptr_term(NEG);			/* mass*ay */
	new_ptr_var(massn);
	new_ptr_var(acyn);
	new_ptr_var(len);
	sprintf(temp,"cos_cm%d",link->num);
	new_ptr_var(temp);
}

/************************************************************
	PROC:	add_t_parent(link)
	DOES:	adds torque applied by parent link
************************************************************/

void add_t_parent(LINK *link)
{
	STR tn;

	sprintf(tn,"t[%d]",link->num-1);

	new_ptr_term(POS);
	new_ptr_var(tn);
}

/************************************************************
	PROC:	add_t_iw(link)
	DOES:	add iw to torque equation
************************************************************/

void add_t_iw(LINK *link)
{
	STR ibn,ddthn;
	int n;

	sprintf(ibn,"INER%d",link->num);
	for (n=1; n<=link->linklist[0]; n++) {
		sprintf(ddthn,"ddth%d",link->linklist[n]);	

		new_ptr_term(NEG);
		new_ptr_var(ibn);
		new_ptr_var(ddthn);
	}
}

/************************************************************
	PROC:	add_t_fix(link)
	DOES:	adds torques due to forces at fixed points
************************************************************/

void add_t_fix(LINK *link)
{
	STR fxfn,fyfn,lenn;
	STR temp;

	sprintf(fxfn,"Fxf%d",link->num);
	sprintf(fyfn,"Fyf%d",link->num);
	sprintf(lenn,"LENFIX%d",link->num);

	new_ptr_term(NEG);		/* torque by fx*lenat */
	new_ptr_var(fxfn);
	new_ptr_var(lenn);
	sprintf(temp,"sin_f%d",link->num);
	new_ptr_var(temp);
	new_ptr_term(POS);		/* torque by fy*lenat */
	new_ptr_var(fyfn);
	new_ptr_var(lenn);
	sprintf(temp,"cos_f%d",link->num);
	new_ptr_var(temp);
}

/************************************************************
	PROC:	add_t_ext(link)
	DOES:	adds torques due to external forces
************************************************************/

void add_t_ext(LINK *link)
{
	STR fxfn,fyfn,lenn;
	MON *mon;
	STR temp;

	if (ROotmon) {
		new_ptr_term(POS);
		sprintf(temp,"Fext[%d]",(link->num-1)*3+2);
		new_ptr_var(temp);
		return;
	}
	for (mon=link->mon; mon; mon=mon->next) {
		sprintf(fxfn,"Fext[%d]",(mon->num-1)*2);
		sprintf(fyfn,"Fext[%d]",(mon->num-1)*2+1);
		sprintf(lenn,"LENMON%d",mon->num);

		new_ptr_term(NEG);		/* torque by fx*lenmon */
		new_ptr_var(fxfn);
		new_ptr_var(lenn);
		sprintf(temp,"sin_m%d",mon->num);
		new_ptr_var(temp);
		new_ptr_term(POS);		/* torque by fy*lenmon */
		new_ptr_var(fyfn);
		new_ptr_var(lenn);
		sprintf(temp,"cos_m%d",mon->num);
		new_ptr_var(temp);
	}
}

/************************************************************
	PROC:	add_t_child(link)
	DOES:	adds torques due to children of link
************************************************************/

void add_t_child(LINK *link)
{
	CHILD *child;
	STR len,tn;
	STR temp;

	child = link->child;
	while (child) {
		sprintf(len,"LENAT%d",child->link->num);
		sprintf(tn,"t[%d]",child->link->num-1);

		new_ptr_term(NEG);		/* torque exerted at pin joint */
		new_ptr_var(tn);
		new_ptr_term(POS);		/* torque by child fx*lenat */
		sprintf(temp,"ptr_fx%d",child->link->num);
		new_ptr_var(temp);
		new_ptr_var(len);
		sprintf(temp,"sin_at%d",child->link->num);
		new_ptr_var(temp);
		new_ptr_term(NEG);		/* torque by child fy*lenat */
		sprintf(temp,"ptr_fy%d",child->link->num);
		new_ptr_var(temp);
		new_ptr_var(len);
		sprintf(temp,"cos_at%d",child->link->num);
		new_ptr_var(temp);
		child = child->next;
	}
}

