/************************************************************
	FILE: com3.c

************************************************************/

#include "defs.h"
#include "globals.h"

/************************************************************
	PROC:	mon_mark(link)
	DOES:	marks any links needing calculation of terms for monitor()
************************************************************/

BOOL mon_mark(LINK *link)
{
	CHILD *c;
	BOOL mon=FALSE;
	BOOL monc;

	for (c=link->child; c; c=c->next) {
		monc = mon_mark(c->link);
		if (monc) mon=TRUE;
	}
	if (link->mon) mon=TRUE;
	link->mon_calc = mon;
	return(mon);
}

/************************************************************
	PROC:	gen_mon_eqns()
	DOES:	generates monitor equations
************************************************************/

void gen_mon_eqns(FILE *file)
{
	fprintf(file,"mon_%s(state,mstate)\n",PRocname);
	fprintf(file,"float *state,*mstate;\n");
	fprintf(file,"{\n");
	Const_eqns.num_eqns=0;		/* start over again */
	rcalc_mon(Rlink);			/* calculate monitor consts */
	mon_local_vars(file);		/* declare local vars */
	const_calc(file);			/* print consts to file */
	fprintf(file,"}\n\n");
}

/************************************************************
	PROC:	mon_local_vars(file)
	DOES:	declare necessary local variables
************************************************************/

void mon_local_vars(FILE *file)
{
	int max_knum = 0;
	char *name;
	int n;

	for (n=0; n<Const_eqns.num_eqns; n++) {		/* for each const eqn */
		name = Const_eqns.eqn[n].label;
		if (strncmp(name,"mstate[",7)) {
			fprintf(file,"\tfloat %s;\n",name);
		}
	}
	fprintf(file,"\n");
}

/************************************************************
	PROC:	rcalc_mon()
	DOES:	calculates monitor constants
************************************************************/

void rcalc_mon(LINK *link)
{
	CHILD *c;

	if (!link->mon_calc) return;		/* not necessary for this link */
	mon_const(link);
	for (c=link->child; c; c=c->next)	/* perform mon calcs for children */
		rcalc_mon(c->link);
}

/************************************************************
	PROC:	mon_const()
	DOES:	calculates intermediate constants to be used in
	       calculating the position and velocity of monitor points
************************************************************/

void mon_const(LINK *link)
{
	MON *mon;
	STR temp;

		/* calc th, dth, etc... */
	new_const_eqn("th",link->num);			/* const eqn for th */
	new_const_term(POS);
	sprintf(temp,"state[%d]",link->num*2+2);
	new_const_var(temp);
	if (link!=Rlink) {
		new_const_term(POS);
		sprintf(temp,"th%d",link->parent);
		new_const_var(temp);
	}

	new_const_eqn("dth",link->num);			/* const eqn for dth */
	new_const_term(POS);
	sprintf(temp,"state[%d]",link->num*2+3);
	new_const_var(temp);
	if (link!=Rlink) {
		new_const_term(POS);
		sprintf(temp,"dth%d",link->parent);
		new_const_var(temp);
	}

	if (link->parent) {
		new_const_eqn("at_angle",link->num);	/* const eqn for at_angle */
		new_const_term(POS);
		sprintf(temp,"th%d",link->parent);
		new_const_var(temp);
		new_const_term(POS);
		sprintf(temp,"A_AT%d",link->num);
		new_const_var(temp);
		
		new_const_eqn("sin_at",link->num);		/* const eqn for sin_at */
		new_const_term(POS);
		sprintf(temp,"sin(at_angle%d)",link->num);
		new_const_var(temp);
		
		new_const_eqn("cos_at",link->num);		/* const eqn for cos_at */
		new_const_term(POS);
		sprintf(temp,"cos(at_angle%d)",link->num);
		new_const_var(temp);
	}

	new_const_eqn("xo",link->num);				/* const eqn for x-origin */
	if (!link->parent) {
		new_const_term(POS);
		new_const_var("state[0]");
	} else {
		new_const_term(POS);
		sprintf(temp,"xo%d",link->parent);
		new_const_var(temp);
		new_const_term(POS);
		sprintf(temp,"LENAT%d",link->num);
		new_const_var(temp);
		sprintf(temp,"cos_at%d",link->num);
		new_const_var(temp);
	}

	new_const_eqn("yo",link->num);				/* const eqn for y-origin */
	if (!link->parent) {
		new_const_term(POS);
		new_const_var("state[2]");
	} else {
		new_const_term(POS);
		sprintf(temp,"yo%d",link->parent);
		new_const_var(temp);
		new_const_term(POS);
		sprintf(temp,"LENAT%d",link->num);
		new_const_var(temp);
		sprintf(temp,"sin_at%d",link->num);
		new_const_var(temp);
	}

	new_const_eqn("vxo",link->num);				/* const eqn for vx-origin */
	if (!link->parent) {
		new_const_term(POS);
		new_const_var("state[1]");
	} else {
		new_const_term(POS);
		sprintf(temp,"vxo%d",link->parent);
		new_const_var(temp);
		new_const_term(NEG);
		sprintf(temp,"LENAT%d",link->num);
		new_const_var(temp);
		sprintf(temp,"sin_at%d",link->num);
		new_const_var(temp);
		sprintf(temp,"dth%d",link->parent);
		new_const_var(temp);
	}

	new_const_eqn("vyo",link->num);				/* const eqn for vy-origin */
	if (!link->parent) {
		new_const_term(POS);
		new_const_var("state[3]");
	} else {
		new_const_term(POS);
		sprintf(temp,"vyo%d",link->parent);
		new_const_var(temp);
		new_const_term(POS);
		sprintf(temp,"LENAT%d",link->num);
		new_const_var(temp);
		sprintf(temp,"cos_at%d",link->num);
		new_const_var(temp);
		sprintf(temp,"dth%d",link->parent);
		new_const_var(temp);
	}

	for (mon=link->mon; mon; mon=mon->next) {	/* const eqn for mon angles */
		new_const_eqn("m_angle",mon->num);			/* m_angle */
		new_const_term(POS);
		sprintf(temp,"th%d",link->num);
		new_const_var(temp);
		new_const_term(POS);
		sprintf(temp,"A_M%d",mon->num);
		new_const_var(temp);

		new_const_eqn("sin_m",mon->num);			/* sin m_angle */
		new_const_term(POS);
		sprintf(temp,"sin(m_angle%d)",mon->num);
		new_const_var(temp);

		new_const_eqn("cos_m",mon->num);			/* cos m_angle */
		new_const_term(POS);
		sprintf(temp,"cos(m_angle%d)",mon->num);
		new_const_var(temp);

		sprintf(temp,"mstate[%d]",(mon->num-1)*4+0);	/* mon x */
		new_const_eqn(temp,0);
		new_const_term(POS);
		sprintf(temp,"xo%d",link->num);
		new_const_var(temp);
		new_const_term(POS);
		sprintf(temp,"LENMON%d",mon->num);
		new_const_var(temp);
		sprintf(temp,"cos_m%d",mon->num);
		new_const_var(temp);

		sprintf(temp,"mstate[%d]",(mon->num-1)*4+2);	/* mon y */
		new_const_eqn(temp,0);
		new_const_term(POS);
		sprintf(temp,"yo%d",link->num);
		new_const_var(temp);
		new_const_term(POS);
		sprintf(temp,"LENMON%d",mon->num);
		new_const_var(temp);
		sprintf(temp,"sin_m%d",mon->num);
		new_const_var(temp);

		sprintf(temp,"mstate[%d]",(mon->num-1)*4+1);	/* mon vx */
		new_const_eqn(temp,0);
		new_const_term(POS);
		sprintf(temp,"vxo%d",link->num);
		new_const_var(temp);
		new_const_term(NEG);
		sprintf(temp,"LENMON%d",mon->num);
		new_const_var(temp);
		sprintf(temp,"sin_m%d",mon->num);
		new_const_var(temp);
		sprintf(temp,"dth%d",link->num);
		new_const_var(temp);

		sprintf(temp,"mstate[%d]",(mon->num-1)*4+3);	/* mon vy */
		new_const_eqn(temp,0);
		new_const_term(POS);
		sprintf(temp,"vyo%d",link->num);
		new_const_var(temp);
		new_const_term(POS);
		sprintf(temp,"LENMON%d",mon->num);
		new_const_var(temp);
		sprintf(temp,"cos_m%d",mon->num);
		new_const_var(temp);
		sprintf(temp,"dth%d",link->num);
		new_const_var(temp);
	}
}
