/************************************************************
	FILE: com4.c

************************************************************/

#include "defs.h"
#include "globals.h"

/************************************************************
	PROC:	mul_term_eqn(term,eqn)
	DOES:	multiplies an equation by a term
************************************************************/

TERM *mul_term_eqn(TERM *term, TERM *eqn)
{
	TERM *new_eqn;
	TERM *termn;
	P *p;

/*
	if (Verbose) {
		printf("Multiplying term by equation:\n");
		preqn("\tterm:",term);
		preqn("\teqn:",eqn);
	}
*/
	new_eqn = eqncpy(eqn);
	termn = new_eqn;
	while (termn) {
		p = term->p;
		while (p) {
			paddstr(termn,p->txt);
			p = p->next;
		}
			/* take care of sign */
		if (term->neg) termn->neg = !termn->neg;
		termn = termn->next;
	}
	return(new_eqn);
}

/************************************************************
	PROC:	place_terms(eqnum,eqn,label)
	DOES:	places terms into matrix
************************************************************/

void place_terms(int eqnum, TERM *eqn, char *label)
{
	BOOL found;
	int var,uvar;
	TERM *term,*newterm;
	P *p;

	Eqns[eqnum] = eqn;		/* save equation */
		/* create equation label */
	strcpy(Label[eqnum],label);
		/* classify each term as either containing an unknown */
		/* or being a constant */
	for (term=eqn; term; term=term->next) {
			/* create a new term */
		newterm = (TERM *) mmalloc(sizeof(TERM));
		newterm->p = (P *) 0;
		newterm->neg = term->neg;
		newterm->next = (TERM *) 0;
			/* look for unknowns in a term */
		for (uvar = -1,p=term->p; p; p=p->next) {
				/* loop thru unknowns */
			for (found=FALSE,var=0; var<Msize; var++) {
				if (!strcmp(Var[var],p->txt)) {	
					if (uvar != -1) 
						printf("Two unknowns in the same term...\n");
					found = TRUE;
					uvar = var;
				}
			}	/* end unknown loop */
			if (!found) {		/* this item in the term is not an unknown */
				paddstr(newterm,p->txt);
			}
		}	/* end within-term loop */
		if (uvar != -1) {
			newterm->next = Eqmat[eqnum][uvar];
			Eqmat[eqnum][uvar] = newterm;
		}
		else {
			newterm->neg = !(newterm->neg);
			newterm->next = Eqconst[eqnum];
			Eqconst[eqnum] = newterm;
		}
	}	/* end term loop */
}		

/************************************************************
	PROC:	stats
	DOES:	returns number of links, fixed-links, and gnd-pts
************************************************************/

void stats()
{
	LINK *link;

	for (link=Link,Nlinks=Flinks=0; link; Nlinks++,link=link->next) 
		if (link->fix) Flinks++;
	Msize = Nlinks + 2*Flinks + 2;
	if (Verbose) {
		printf("STATS: %d links, %d fixed links, matrix size=%d\n",
		Nlinks,Flinks,Msize);
	}
}

/************************************************************
	PROC:	gen_vars
	DOES:	generates names of unknowns
************************************************************/

void gen_vars()
{
	LINK *link;
	int var = 2;
	int fvar = 0;
	int var_num;

		/* generate unknowns for root link */
	sprintf(Var[0],"arx");
	sprintf(Var[1],"ary");
	for (link=Link;link;link=link->next) {
		if (!INverse) 
			sprintf(Var[var],"ddth%d",link->num);
		else
			sprintf(Var[var],"t[%d]",link->num-1);
		var++;
		/* generate unknowns for fixed pts */
		if (link->fix) {
			var_num = Nlinks + 2*fvar + 2;
			sprintf(Var[var_num],"Fxf%d",link->num);
			sprintf(Var[var_num+1],"Fyf%d",link->num);
			fvar++;
		}
	}
		/* initialize translate table */
	init_trans();
}

/************************************************************
	PROC:	prVars()
	DOES:	prints out list of unknowns
************************************************************/

void prVars()
{
	int var;

	printf("Unknown vars:\n");
	for (var=0; var<Msize; var++)
		printf("%2d: %s\n",var,Var[var]);
}

/************************************************************
	PROC:	prLabels()
	DOES:	prints equation labels
************************************************************/

void prLabels()
{
	int label;

	printf("Equation labels:\n");
	for (label=0; label<Msize; label++)
		printf("%2d: %s\n",label,Label[label]);
}

/************************************************************
	PROC:	pr_eqn_mat()
	DOES:	prints equation matrix
************************************************************/

void pr_eqn_mat()
{
	int row,col;
	STR termLabel;

		/* print out matrix contents */	
	for (row=0; row<Msize; row++) {
		for (col=0; col<Msize; col++) {
			sprintf(termLabel,"A:%d:%d",row,col);
			preqn(termLabel,Eqmat[row][col]);
		}
	}
		/* print out constants */
	for (row=0; row<Msize; row++) {
		sprintf(termLabel,"b:%d",row);
		preqn(termLabel,Eqconst[row]);
	}
}

/************************************************************
	PROC:	pr_Eqns
	DOES:	prints out an equation-set
************************************************************/

void pr_Eqns(EQNSET *eqnset, char *name)
{
	int n;

	printf("--------------------------------\n");
	printf("Equation set %s contains %d equations:\n",name,eqnset->num_eqns);
	for (n=0;n<eqnset->num_eqns; n++)
		preqn(eqnset->eqn[n].label,eqnset->eqn[n].terms);
}
