/************************************************************
	FILE: cm6.cpp

************************************************************/

#include "defs.h"
#include "globals.h"

/************************************************************
	PROC:	newterm(eqn)
	DOES:	adds a new term to a list of terms (eqn)
			-returns a pointer to the new term
************************************************************/

TERM *newterm(TERM *eqn)
{
	TERM *temp;

	temp = eqn;
	eqn = (TERM *) mmalloc(sizeof(TERM));
	eqn->next = temp;
	eqn->neg = FALSE;
	eqn->p = (P *) NULL;
	return(eqn);
}

/************************************************************
	PROC:	calc_accel
	DOES:	Symbolically calculates the accelerations for each link:
				-center of mass
				-fixed points
				-children
************************************************************/

void calc_accel(LINK *link)
{
	calc_acm(link);				/* find center of mass acc */
	calc_afix(link);			/* find fixed pt acc */
	calc_achild(link);			/* find child acc */
}

/************************************************************
	PROC:	rcalclist
	DOES:	finds relation of link to root link; this is used
			to form strings with the absolute angles and angular
			velocities for the link (th_list and dth_list, respectively)
************************************************************/

void rcalclist(LINK *link, int linklist[])
{
	int linklist2[MAX_LINKS];
	int n;
	int link_num;
	CHILD *c;

	link_num = link->num;
	linklist[++linklist[0]] = link_num;
	for (n=0;n<=linklist[0];n++) {
		link->linklist[n] = linklist[n];
	}
	buildlist(link);
	for (c=link->child; c; c=c->next) {
		for (n=0;n<=link->linklist[0];n++) {		/* copy current list */
			linklist2[n] = link->linklist[n];
		}
		rcalclist(c->link,linklist2);			/* calc link list */
	}
}

/************************************************************
	PROC:	preqn
	DOES:	prints an eqn
************************************************************/

void preqn(char *eqn_name, TERM *eqn)
{
	TERM *term;

	if (!eqn) return;
	term = eqn;
	printf("%s = ",eqn_name);
	while(term) {
		prterm(term);
		term = term->next;
	}			/* end while(term) */
	printf("\n");
}

/************************************************************
	PROC:	prterm()
	DOES:	prints a term
************************************************************/

void prterm(TERM *term)
{
	P *p;

	printf("\n\t%c",term->neg?'-':'+');
	p = term->p;
	if (p) {
		printf("%s",p->txt);
		p = p->next;
	} else printf("1");
	while (p) {
		printf("*%s",p->txt);
		p = p->next;
	}
}

/************************************************************
	PROC:	calc_acm(link)
	DOES:	calculates (symbolically) the acceleration of the
			center of mass of a link
************************************************************/

void calc_acm(LINK *link)
{
	STR temp;
	int n;

	new_ptr_eqn("ptr_acx",link->num);			/* new ptr eqn */
	new_ptr_term(POS);
	sprintf(temp,"ptr_abx%d",link->num);		/* acc of base */
	new_ptr_var(temp);
	new_ptr_term(NEG);							/* rww term */
	sprintf(temp,"dth%d",link->num);
	new_ptr_var(temp);
	new_ptr_var(temp);
	sprintf(temp,"LENCM%d",link->num);
	new_ptr_var(temp);
	sprintf(temp,"cos_cm%d",link->num);
	new_ptr_var(temp);
	for (n=1; n<=link->linklist[0]; n++) {		/* ralpha term */
		new_ptr_term(NEG);
		sprintf(temp,"sin_cm%d",link->num);
		new_ptr_var(temp);
		sprintf(temp,"ddth%d",link->linklist[n]);
		new_ptr_var(temp);
		sprintf(temp,"LENCM%d",link->num);
		new_ptr_var(temp);
	}

	new_ptr_eqn("ptr_acy",link->num);		/* new ptr eqn */
	new_ptr_term(POS);
	sprintf(temp,"ptr_aby%d",link->num);		/* acc of base */
	new_ptr_var(temp);
	new_ptr_term(NEG);						/* rww term */
	sprintf(temp,"dth%d",link->num);
	new_ptr_var(temp);
	new_ptr_var(temp);
	sprintf(temp,"LENCM%d",link->num);
	new_ptr_var(temp);
	sprintf(temp,"sin_cm%d",link->num);
	new_ptr_var(temp);
	for (n=1; n<=link->linklist[0]; n++) {		/* ralpha term */
		new_ptr_term(POS);
		sprintf(temp,"cos_cm%d",link->num);
		new_ptr_var(temp);
		sprintf(temp,"ddth%d",link->linklist[n]);
		new_ptr_var(temp);
		sprintf(temp,"LENCM%d",link->num);
		new_ptr_var(temp);
	}
}

/************************************************************
	PROC:	calc_achild(link)
	DOES:	calculates accelerations for children of a link
************************************************************/

void calc_achild(LINK *link)
{
	CHILD *child;
	STR temp;
	int n;

	child = link->child;
	while (child) {						/* calculate acc for child base */
		new_ptr_eqn("ptr_abx",child->link->num);	/* new ptr eqn */
		new_ptr_term(POS);
		sprintf(temp,"ptr_abx%d",link->num);		/* acc of base */
		new_ptr_var(temp);
		new_ptr_term(NEG);						/* rww term */
		sprintf(temp,"dth%d",link->num);
		new_ptr_var(temp);
		new_ptr_var(temp);
		sprintf(temp,"LENAT%d",child->link->num);
		new_ptr_var(temp);
		sprintf(temp,"cos_at%d",child->link->num);
		new_ptr_var(temp);
		for (n=1; n<=link->linklist[0]; n++) {		/* ralpha term */
			new_ptr_term(NEG);
			sprintf(temp,"sin_at%d",child->link->num);
			new_ptr_var(temp);
			sprintf(temp,"ddth%d",link->linklist[n]);
			new_ptr_var(temp);
			sprintf(temp,"LENAT%d",child->link->num);
			new_ptr_var(temp);
			}
		
		new_ptr_eqn("ptr_aby",child->link->num);	/* new ptr eqn */
		new_ptr_term(POS);
		sprintf(temp,"ptr_aby%d",link->num);		/* acc of base */
		new_ptr_var(temp);
		new_ptr_term(NEG);						/* rww term */
		sprintf(temp,"dth%d",link->num);
		new_ptr_var(temp);
		new_ptr_var(temp);
		sprintf(temp,"LENAT%d",child->link->num);
		new_ptr_var(temp);
		sprintf(temp,"sin_at%d",child->link->num);
		new_ptr_var(temp);
		for (n=1; n<=link->linklist[0]; n++) {		/* ralpha term */
			new_ptr_term(POS);
			sprintf(temp,"cos_at%d",child->link->num);
			new_ptr_var(temp);
			sprintf(temp,"ddth%d",link->linklist[n]);
			new_ptr_var(temp);
			sprintf(temp,"LENAT%d",child->link->num);
			new_ptr_var(temp);
		}

			/* calculate other accelerations for link */
		calc_accel(child->link);
		child = child->next;
	}				/* end while(child) */
}

/************************************************************
	PROC:	calc_afix(link)
	DOES:	calculates accelerations for fixed points on link
************************************************************/

void calc_afix(LINK *link)
{
	STR temp;		/* temp buffer for strings */
	int n;			/* counter */

	if (!link->fix) return;
	new_ptr_eqn("ptr_afx",link->num);			/* new ptr eqn */
	new_ptr_term(POS);
	sprintf(temp,"ptr_abx%d",link->num);		/* acc of base */
	new_ptr_var(temp);
	new_ptr_term(NEG);						/* rww term */
	sprintf(temp,"dth%d",link->num);
	new_ptr_var(temp);
	new_ptr_var(temp);
	sprintf(temp,"LENFIX%d",link->num);
	new_ptr_var(temp);
	sprintf(temp,"cos_f%d",link->num);
	new_ptr_var(temp);
	for (n=1; n<=link->linklist[0]; n++) {		/* ralpha term */
		new_ptr_term(NEG);
		sprintf(temp,"sin_f%d",link->num);
		new_ptr_var(temp);
		sprintf(temp,"ddth%d",link->linklist[n]);
		new_ptr_var(temp);
		sprintf(temp,"LENFIX%d",link->num);
		new_ptr_var(temp);
	}

	new_ptr_eqn("ptr_afy",link->num);			/* new ptr eqn */
	new_ptr_term(POS);
	sprintf(temp,"ptr_aby%d",link->num);		/* acc of base */
	new_ptr_var(temp);
	new_ptr_term(NEG);						/* rww term */
	sprintf(temp,"dth%d",link->num);
	new_ptr_var(temp);
	new_ptr_var(temp);
	sprintf(temp,"LENFIX%d",link->num);
	new_ptr_var(temp);
	sprintf(temp,"sin_f%d",link->num);
	new_ptr_var(temp);
	for (n=1; n<=link->linklist[0]; n++) {		/* ralpha term */
		new_ptr_term(POS);
		sprintf(temp,"cos_f%d",link->num);
		new_ptr_var(temp);
		sprintf(temp,"ddth%d",link->linklist[n]);
		new_ptr_var(temp);
		sprintf(temp,"LENFIX%d",link->num);
		new_ptr_var(temp);
	}
}
