/************************************************************
	FILE:

************************************************************/

#include "defs.h"
#include "globals.h"

/************************************************************
	PROC:	calc_force(link)
	DOES:	calculates expression for force on parent link in
			terms of masses and accelerations
************************************************************/

void calc_force(LINK *link)
{
	CHILD *child;

		/* calculate forces for all children */
	for (child=link->child; child; child=child->next) {
		calc_force(child->link);
	}
	calc_forcex(link);
	calc_forcey(link);
}

/************************************************************
	PROC:	term_add(terma,termb)
	DOES:	adds termb to terma
************************************************************/

TERM *term_add(TERM *terma, TERM *termb)
{
	TERM *new_termb,*temp;

	new_termb = eqncpy(termb);
	temp = terma;
	if (temp) {
		while(temp->next) temp = temp->next;
		temp->next = new_termb;
		return(terma);
	}
	else return(new_termb);
}

/************************************************************
	PROC:	term_sub(terma,termb)
	DOES:	subtracts termb from terma
************************************************************/

TERM *term_sub(TERM *terma, TERM *termb)
{
	TERM *new_termb,*temp;

		/* change sign of termb */
	new_termb = eqncpy(termb);
	temp = new_termb;
	while(temp) {
		temp->neg = !temp->neg;
		temp = temp->next;
	}
		/* add to terma */
	temp = terma;
	if (temp) {
		while(temp->next) temp = temp->next;
		temp->next = new_termb;
		return(terma);
	}
	else return(new_termb);
}

/************************************************************
	PROC:	calc_forcex(link)
	DOES:	calculates forcex exerted by parent on link
************************************************************/

void calc_forcex(LINK *link)
{
	CHILD *child;
	MON *mon;
	STR temp;

	new_ptr_eqn("ptr_fx",link->num);		/* create ptr eqn */
	new_ptr_term(POS);					/* mass*acc term */
	sprintf(temp,"MASS%d",link->num);
	new_ptr_var(temp);
	sprintf(temp,"ptr_acx%d",link->num);
	new_ptr_var(temp);
	if (link->fix) {					/* fixed force */
		new_ptr_term(NEG);
		sprintf(temp,"Fxf%d",link->num);
		new_ptr_var(temp);
	}
	if (ROotmon) {
		new_ptr_term(NEG);
		sprintf(temp,"Fext[%d]",(link->num-1)*3);
		new_ptr_var(temp);
	} else {
		for (mon=link->mon; mon; mon=mon->next) {	/* external forces */
			new_ptr_term(NEG);
			sprintf(temp,"Fext[%d]",(mon->num-1)*2);
			new_ptr_var(temp);
		}
	}
	for (child=link->child; child; child=child->next) {	/* child forces */
		new_ptr_term(POS);
		sprintf(temp,"ptr_fx%d",child->link->num);
		new_ptr_var(temp);
	}
}

/************************************************************
	PROC:	calc_forcey(link)
	DOES:	calculates forcey exerted by parent on link
************************************************************/

void calc_forcey(LINK *link)
{
	CHILD *child;
	MON *mon;
	STR temp;

	new_ptr_eqn("ptr_fy",link->num);		/* create ptr eqn */
	new_ptr_term(POS);					/* mass*acc term */
	sprintf(temp,"MASS%d",link->num);
	new_ptr_var(temp);
	sprintf(temp,"ptr_acy%d",link->num);
	new_ptr_var(temp);
	if (link->fix) {					/* fixed force */
		new_ptr_term(NEG);
		sprintf(temp,"Fyf%d",link->num);
		new_ptr_var(temp);
	}
	if (ROotmon) {
		new_ptr_term(NEG);
		sprintf(temp,"Fext[%d]",(link->num-1)*3+1);
		new_ptr_var(temp);
	} else {
		for (mon=link->mon; mon; mon=mon->next) {	/* external forces */
			new_ptr_term(NEG);
			sprintf(temp,"Fext[%d]",(mon->num-1)*2+1);
			new_ptr_var(temp);
		}
	}
	new_ptr_term(NEG);							/* gravity */
	sprintf(temp,"MASS%d",link->num);
	new_ptr_var(temp);
	new_ptr_var(GRAVITY);
	for (child=link->child; child; child=child->next) {	/* child forces */
		new_ptr_term(POS);
		sprintf(temp,"ptr_fy%d",child->link->num);
		new_ptr_var(temp);
	}
}
