/************************************************************
	FILE:	com8.c

************************************************************/

#include "defs.h"
#include "globals.h"

static STR name1[MAX_LINKS];
static STR name2[MAX_LINKS];

/************************************************************
	PROC:	new_const_eqn()
	DOES:	creates a new constant equation
************************************************************/

void new_const_eqn(STR str, int num)
{
	int eqnum;
	EQN *eqn;
	extern TERM *eqncpy();

	eqnum = Const_eqns.num_eqns++;		/* get current eqn num */
	eqn = &(Const_eqns.eqn[eqnum]);			/* get ptr to eqn */
	if (num!=0)							/* make label */
		sprintf(eqn->label,"%s%d",str,num);
	else
		sprintf(eqn->label,"%s",str);
	eqn->terms = (TERM *) NULL;
	eqn->flat = FALSE;
}

/************************************************************
	PROC:	new_const_term()
	DOES:	adds a new term to a pointer equation
************************************************************/

void new_const_term(int sign)
{
	EQN *eqn;

	eqn = &(Const_eqns.eqn[Const_eqns.num_eqns-1]);
	eqn->terms = newterm(eqn->terms);
	eqn->terms->neg = sign;
}

/************************************************************
	PROC:	new_const_var()
	DOES:	adds a new var to the first term in a pointer eqn
************************************************************/

void new_const_var(char *str)
{
	EQN *eqn;

	eqn = &(Const_eqns.eqn[Const_eqns.num_eqns-1]);
	paddstr(eqn->terms,str);
}

/************************************************************
	PROC:	new_ptr_eqn()
	DOES:	creates a new pointer equation
************************************************************/

void new_ptr_eqn(STR str, int num)
{
	int eqnum;
	EQN *eqn;
	extern TERM *eqncpy();

	eqnum = Ptr_eqns.num_eqns++;		/* get current eqn num */
	eqn = &(Ptr_eqns.eqn[eqnum]);			/* get ptr to eqn */
	if (num!=0)							/* make label */
		sprintf(eqn->label,"%s%d",str,num);
	else
		sprintf(eqn->label,"%s",str);
	eqn->terms = (TERM *) NULL;
	eqn->flat = FALSE;
}

/************************************************************
	PROC:	new_ptr_term()
	DOES:	adds a new term to a pointer equation
************************************************************/

void new_ptr_term(int sign)
{
	EQN *eqn;

	eqn = &(Ptr_eqns.eqn[Ptr_eqns.num_eqns-1]);
	eqn->terms = newterm(eqn->terms);
	eqn->terms->neg = sign;
}

/************************************************************
	PROC:	new_ptr_var()
	DOES:	adds a new var to the first term in a pointer eqn
************************************************************/

void new_ptr_var(char *str)
{
	EQN *eqn;

	eqn = &(Ptr_eqns.eqn[Ptr_eqns.num_eqns-1]);
	translate(str);
	paddstr(eqn->terms,str);
}

/************************************************************
	PROC:	translate(str)
	DOES:	translates the given string, if necessary
************************************************************/

void translate(char *str)
{
	int count=0;

	while (name1[count][0]) {
		if (!strcmp(name1[count],str)) {
			strcpy(str,name2[count]);
			return;
		}
		count++;
	}
}

/************************************************************
	PROC:	init_trans()
	DOES:	initializes the translate table
************************************************************/

void init_trans()
{
	int n;

	for (n=0; n<Msize; n++) {
		strcpy(name1[n],Var[n]);
		sprintf(name2[n],"unk%d",n);
	}
	n=Msize;
	sprintf(name1[n],"ptr_abx%d",Rlink->num);
	sprintf(name2[n],"unk0");
	n++;
	sprintf(name1[n],"ptr_aby%d",Rlink->num);
	sprintf(name2[n],"unk1");
}

/************************************************************
	PROC:	reduce()
	DOES:	collects constants in equations
************************************************************/

void reduce()
{
	int n;
	TERM **pterm;
	BOOL first;
	STR name;
	TERM *term = (TERM *) NULL;

	for (n=0; n<Ptr_eqns.num_eqns; n++) {		/* for each eqn */
		first = TRUE;
		pterm = &(Ptr_eqns.eqn[n].terms);		/* get first term */
		while (*pterm) {						/* for each term */
			if (term_type(*pterm)==CONST_TERM) {
				if (first) {					/* if first term */
					sprintf(name,"k[%d]",Kcount);	/* create a new const */
					Kcount++;					/* inc const count */
					new_const_eqn(name,0);		/* create eqn */
					term = newterm(term);		/* create new term */
					paddstr(term,name);			/* add var */
					insert_term(pterm,term);	/* insert term */
					pterm = &(term->next);		/* advance pterm */
					first = FALSE;
				}
				term2const(pterm);				/* transfer to const */
			}
			else if (*pterm)
				pterm = &((*pterm)->next);			/* next term */
		}
		pterm = &(Ptr_eqns.eqn[n].terms);		/* handle multiple const */
		while (*pterm) {						/* for each term */
			mfix(*pterm);						/* fix it if nec. */
			pterm = &((*pterm)->next);			/* next term */
		}
	}
}

/************************************************************
	PROC:	term_type()
	DOES:	classifies the type of a product term
************************************************************/

int term_type(TERM *term)
{
	P *p;

	for (p=term->p; p; p=p->next) {
		if (!strncmp(p->txt,"unk",3))
			return(UNK_TERM);
		 if (!strncmp(p->txt,"ptr",3))
			 return(PTR_TERM);
	}
	return(CONST_TERM);
}

/************************************************************
	PROC:	insert_term()
	DOES:	inserts a new term into a list of terms
************************************************************/

void insert_term(TERM **pterm, TERM *term)
{
	TERM *temp;

	temp = *pterm;		/* insert term */
	*pterm = term;
	term->next = temp;
}

/************************************************************
	PROC:	term2const(pterm)
	DOES:	moves the given term to the current const equation,
			and deletes it from it's current position
************************************************************/

void term2const(TERM **pterm)
{
	EQN *eqn;
	TERM *temp1,*temp2;

	eqn = &(Const_eqns.eqn[Const_eqns.num_eqns-1]);
	temp1 = (*pterm)->next;
	temp2 = eqn->terms;
	eqn->terms = *pterm;			/* insert into const eqn */
	eqn->terms->next = temp2;
	*pterm = temp1;					/* remove from ptr eqn */
}

/************************************************************
	PROC:	mfix(term)
	DOES:	fixes terms having multiple constants in front
************************************************************/

void mfix(TERM *term)
{
	P *p,*oldp;
	int count=0;
	STR name;

	for (p=term->p; p; p=p->next)
		count++;
	if (count>2) {
		sprintf(name,"k[%d]",Kcount);		/* create new const */
		Kcount++;						/* inc const count */
		new_const_eqn(name,0);			/* create eqn */
		new_const_term(term->neg?NEG:POS);			/* create a term */
		term->neg = FALSE;				/* get rid of negative sign */
		for (p=term->p; p; p=p->next) {
			if (!strncmp(p->txt,"unk",3) || !strncmp(p->txt,"ptr",3)) {
				oldp = p;
				continue;				/* leave it alone */
			}
			new_const_var(p->txt);		/* add to constant term */
			if (p==term->p)
				term->p = p->next;
			else
				oldp->next = p->next;
		}
		paddstr(term,name);				/* add k to eqn */
	}
}
