/************************************************************
	FILE: cm9.c

************************************************************/

#include "defs.h"
#include "globals.h"

EQN *Solve_eqns[MAX_LINKS];
STR Solve_name[MAX_LINKS];

/************************************************************
	PROC:	find_solve_eqns()
	DOES:	creates pointers to solve equations
************************************************************/

void find_solve_eqns()
{
	int n=0;
	LINK *link;

	sprintf(Solve_name[n],"ptr_fx%d",Rlink->num);		/* fx = 0 for root */
	Solve_eqns[n] = eqn_find(Solve_name[n]);
	n++;
	sprintf(Solve_name[n],"ptr_fy%d",Rlink->num);		/* fy = 0 for root */
	Solve_eqns[n] = eqn_find(Solve_name[n]);
	n++;
	for (link=Link; link; link=link->next) {		/* T=Iw for each link */
		sprintf(Solve_name[n],"ptr_teqn%d",link->num);
		Solve_eqns[n] = eqn_find(Solve_name[n]);
		n++;
	}
	for (link=Link; link; link=link->next) {		/* fixed pt eqns */
		if (link->fix) {
			sprintf(Solve_name[n],"ptr_afx%d",link->num);
			Solve_eqns[n] = eqn_find(Solve_name[n]);
			n++;
			sprintf(Solve_name[n],"ptr_afy%d",link->num);
			Solve_eqns[n] = eqn_find(Solve_name[n]);
			n++;
		}
	}
}

/************************************************************
	PROC:	eqn_find()
	DOES:	finds an equation of the given name
************************************************************/

EQN *eqn_find(STR name)
{
	int n;

	for (n=0; n<Ptr_eqns.num_eqns; n++) {		/* for each eqn */
		if (!strcmp(Ptr_eqns.eqn[n].label,name))
			return(&(Ptr_eqns.eqn[n]));				/* found it */
	}
	printf("eqn_find: Eqn '%s' not found\n",name);
	return(NULL);
}

/************************************************************
	PROC:	flat_solve()
	DOES:	flattens all the sub-equations of the solve equations
************************************************************/

void flat_solve()
{
	EQN *eqn,*sub_eqn;
	int n;
	STR ptr_name;
	TERM *term;

	for (n=0; n<Msize; n++) {				/* for each solve eqn */
		eqn = Solve_eqns[n];				/* get equation */
/*
		printf("flat_solve: eqn '%s'\n",Solve_name[n]);
*/
		for (term=eqn->terms; term; term=term->next) {	/* for each term */
			if (term_type(term)==PTR_TERM) {			/* if ptr term */
				extract_ptr(term,ptr_name);			/* get ptr name */
				sub_eqn = eqn_find(ptr_name);			/* get ptr eqn */
				flatten(sub_eqn);				/* flatten it */
			}
		}
	}
}

/************************************************************
	PROC:	extract_ptr()
	DOES:	extracts the ptr var from a ptr term
************************************************************/

void extract_ptr(TERM *term, STR var)
{
	P *p;

	for (p=term->p; p; p=p->next) {
		if (!strncmp(p->txt,"ptr",3)) {
			strcpy(var,p->txt);
			return;
		}
	}
	printf("extract_ptr: ERROR, var '%s' not found\n",var);
}

/************************************************************
	PROC:	extract_unk()
	DOES:	extracts the unk var from a unk term
************************************************************/

void extract_unk(TERM *term, STR var)
{
	P *p;

	for (p=term->p; p; p=p->next) {
		if (!strncmp(p->txt,"unk",3)) {
			strcpy(var,p->txt);
			return;
		}
	}
	printf("extract_unk: ERROR, var '%s' not found\n",var);
}

/************************************************************
	PROC:	flatten()
	DOES:	flattens an equation
************************************************************/

void flatten(EQN *eqn)
{
	int n;
	TERM *term;
	EQN *sub_eqn;
	STR ptr_name;			/* name of sub-eqn ptr */
	STR uvar;				/* name of unknown var */

	if (eqn->flat) return;
	for (term=eqn->terms; term; term=term->next) {	/* flatten subeqns */
		if (term_type(term)==PTR_TERM) {
			extract_ptr(term,ptr_name);
			sub_eqn = eqn_find(ptr_name);
			flatten(sub_eqn);
		}
	}
/*
	printf("flatten: processing terms of '%s'\n",eqn->label);
*/
	for (n=0; n<Msize; n++) {		/* for each unknown */
		strcpy(uvar,Var[n]);
		translate(uvar);
		flat_fix(eqn,uvar);		/* make eqn flat for that var */
	}
	flat_fix(eqn,"const");			/* make eqn flat for const */
	term_ptr_rem(eqn);				/* remove ptr terms */
	eqn->flat = TRUE;
}

/************************************************************
	PROC:	flat_fix()
	DOES:	makes an equation flat w.r.t. a given variable
************************************************************/

void flat_fix(EQN *eqn, STR var)
{
	EQN *new_eqn,*ceqn;
	STR name;				/* name of new const */

/*
	printf("flat_fix: '%s' wrt '%s'\n",eqn->label,var);
	printf("flat_fix: eqn before=\n");
	preqn(eqn->label,eqn->terms);
*/
	new_eqn  = (EQN *) malloc(sizeof(EQN));	/* create new eqn */
	new_eqn->flat = FALSE;
	new_eqn->terms = (TERM *) NULL;
	search_eqn(var,eqn,new_eqn);		/* get the terms */
	if (simple_eqn(new_eqn)) {
		if (!new_eqn->terms) return;
		if (local_term(eqn,var)) return;
		paddstr(new_eqn->terms,var);		/* add unknown var */
		new_eqn->terms->next = eqn->terms;	/* insert term into eqn */
		eqn->terms = new_eqn->terms;
		return;
	}
	sprintf(name,"k[%d]",Kcount);			/* create new const */
	Kcount++;
	new_const_eqn(name,0);				/* create new const eqn */
	ceqn = &(Const_eqns.eqn[Const_eqns.num_eqns-1]);
	ceqn->terms = new_eqn->terms;		/* save it */
	term_var_rem(eqn,var);				/* remove terms containing var */
	term_var_add(eqn,var,name);			/* add constant term back in */
}

/************************************************************
	PROC:	local_term()
	DOES:	Sees if a term is local
************************************************************/

BOOL local_term(EQN *eqn, STR var)
{
	TERM *term;

	for (term=eqn->terms; term; term=term->next) {
		if (term_match(term,var)) return(TRUE);
	}
	return(FALSE);
}

/************************************************************
	PROC:	pause()
	DOES:	waits for a kbd return
************************************************************/

void pause()
{
	STR temp;

	printf("Cont?");
	gets(temp);
}

/************************************************************
	PROC:	search_eqn()
	DOES:	copies terms in eqn having var to neweqn
************************************************************/

void search_eqn(STR var, EQN *eqn, EQN *new_eqn)
{
	TERM *term;
	int tt;			/* term type */
	int stt;		/* sub-term type */
	STR ptr_name;	/* name of subeqn */
	EQN *seqn;		/* sub equation */
	TERM *sterm;	/* sub term */

	for (term=eqn->terms; term; term=term->next) {	/* for every term */
		tt = term_type(term);
		if (tt==PTR_TERM) {
			extract_ptr(term,ptr_name);		/* get name of ptr */
			seqn = eqn_find(ptr_name);		/* get eqn */
			for (sterm=seqn->terms; sterm; sterm=sterm->next) {	/* each sterm */
				stt = term_type(sterm);		/* get sub-term type */
				if (stt==PTR_TERM)
					printf("search_eqn: ERROR\n");
				if (term_match(sterm,var))
					add_kk_term(var,term,sterm,new_eqn);
			}
		} else if (term_match(term,var))
			add_k_term(var,term,new_eqn);
	}
}

/************************************************************
	PROC:	add_k_term()
	DOES:	adds a k term when flattening
************************************************************/

void add_k_term(STR var, TERM *term, EQN *new_eqn)
{
	P *p;

	new_eqn->terms = newterm(new_eqn->terms);	/* add new term */
	new_eqn->terms->neg = term->neg;			/* calc sign */
	for (p=term->p; p; p=p->next) {
		if (strncmp(p->txt,"unk",3))
			paddstr(new_eqn->terms,p->txt);
	}
}

/************************************************************
	PROC:	add_kk_term()
	DOES:	adds a k1*k2 term where necessary when flattening
************************************************************/

void add_kk_term(STR var, TERM *term, TERM *sterm, EQN *new_eqn)
{
	P *p;

	new_eqn->terms = newterm(new_eqn->terms);	/* add new term */
	if ((term->neg && !sterm->neg) || (!term->neg && sterm->neg))
		new_eqn->terms->neg = TRUE;				/* calc sign */
	for (p=term->p; p; p=p->next) {
		if (strncmp(p->txt,"ptr",3)) {			/* if not the ptr */
			paddstr(new_eqn->terms,p->txt);		/* add it */
		}
	}
	for (p=sterm->p; p; p=p->next) {
		if (strncmp(p->txt,"unk",3))			/* if not the unk */
			paddstr(new_eqn->terms,p->txt);		/* add it */
	}
}

/************************************************************
	PROC:	term_match()
	DOES:	finds out if a term and a var are of matching types
************************************************************/

BOOL term_match(TERM *term, STR var)
{
	int tt;
	STR name;

	tt = term_type(term);
	if (tt==CONST_TERM && !strcmp(var,"const"))
		return(TRUE);
	if (tt==UNK_TERM) {
		extract_unk(term,name);
		if (!strcmp(var,name))
			return(TRUE);
	}
	return(FALSE);
}

/************************************************************
	PROC:	term_ptr_rem()
	DOES:	removes ptr terms
************************************************************/

void term_ptr_rem(EQN *eqn)
{
	TERM **pterm;	/* ptr to term */
	int tt;			/* term type */

	pterm = &(eqn->terms);
	while (*pterm) {				/* for each term */
		tt = term_type(*pterm);		/* get term type */
		if (tt==PTR_TERM) {			/* if a ptr term */
			rm_term(pterm);			/* get rid of it */
			continue;
		}
		pterm = &((*pterm)->next);
	}
}

/************************************************************
	PROC:	term_var_rem()
	DOES:	removes terms containing a particular var from an eqn
************************************************************/

void term_var_rem(EQN *eqn, STR var)
{
	TERM **pterm;		/* pointer to term ptr */
	BOOL cons;			/* looking for const term? */
	STR term_var;		/* name of term var */
	int tt;				/* term type */

	cons = !strcmp(var,"const");
	pterm = &(eqn->terms);
	while (*pterm) {
		tt = term_type(*pterm);			/* get term type */
		if (cons && tt==CONST_TERM) {	/* const to be removed */
			rm_term(pterm);
			continue;
		}
		if (tt==UNK_TERM) {
			extract_unk(*pterm,term_var);
			if (!strcmp(var,term_var)) {
				rm_term(pterm);
				continue;
			}
		}
		pterm = &((*pterm)->next);
	}
}

/************************************************************
	PROC:	simple_eqn
	DOES:	determines if the equation is too simple so as to
			not be worthwhile an additional constant
************************************************************/

BOOL simple_eqn(EQN *eqn)
{
	TERM *term;
	int count=0;
	int pcount=0;
	P *p;

	if (!eqn->terms) return(TRUE);		/* no terms */
	for (term=eqn->terms; term; term=term->next) count++;
	if (count>1) return(FALSE);
	for (p=eqn->terms->p; p; p=p->next)  pcount++;
	if (term_type(eqn->terms)==CONST_TERM) {
		if (pcount>1) return(FALSE);
	} else {
		if (pcount>2) return(FALSE);
	}
	return(TRUE);				/* a simple term */
}

/************************************************************
	PROC:	term_var_add()
	DOES:	adds a k*var term, created by flattening an eqn
************************************************************/

void term_var_add(EQN *eqn, STR var, STR name)
{
	eqn->terms = newterm(eqn->terms);		/* create a new term */
	paddstr(eqn->terms,name);				/* add constant */
	if (strcmp(var,"const"))				/* if not "const", add var */
		paddstr(eqn->terms,var);
}

/************************************************************
	PROC:	rm_term()
	DOES:	removes a term from an eqn
************************************************************/

void rm_term(TERM **pterm)
{
	*pterm = (*pterm)->next;
}

/************************************************************
	PROC:	gem()
	DOES:	generates the equation matrix
************************************************************/

void gem()
{
	int n;			/* eqn counter */
	int n2;			/* unknown counter */
	EQN *eqn;		/* solve equation */
	STR uvar;		/* unknown var */
	EQN *new_eqn;	/* new equation */

	printf("gem: Generating equation matrix...\n");
	for (n=0; n<Msize; n++) {				/* for each equation */
		eqn = Solve_eqns[n];				/* find equation */
		for (n2=0; n2<Msize; n2++) {			/* for each unknown */
			strcpy(uvar,Var[n2]);				/* get name */
			translate(uvar);
			new_eqn = (EQN *) malloc(sizeof(EQN));	/* create new eqn */
			new_eqn->terms = (TERM *) NULL;
			search_eqn(uvar,eqn,new_eqn);		/* get terms */
			Eqmat[n][n2] = new_eqn->terms;
		}
		new_eqn = (EQN *) malloc(sizeof(EQN));	/* create new eqn */
		new_eqn->terms = (TERM *) NULL;
		search_eqn("const",eqn,new_eqn);
		negate(new_eqn);
		Eqconst[n] = new_eqn->terms;
	}
}

/************************************************************
	PROC:	negate()
	DOES:	changes the sign on all terms in an eqn
************************************************************/

void negate(EQN *eqn)
{
	TERM *term;

	for (term=eqn->terms; term; term=term->next)
		term->neg = !term->neg;
}
