/************************************************************
	FILE: defs.h
		Michiel van de Panne, University of Toronto
************************************************************/

#ifdef WIN32
#include <windows.h>
#endif
#include <stdio.h>
#include <string.h>
#define TRUE 1
#define FALSE 0
#define VECSIZE 4
#define MAX_ARGS 15
#define NEG 1
#define POS 0
#define CONST_TERM 1
#define UNK_TERM 2
#define PTR_TERM 3

typedef int BOOL;
typedef float FVEC[VECSIZE];
typedef int IVEC[VECSIZE];
#define STRLEN 150
typedef char STR[STRLEN];
typedef void (*PFI)(int argc, STR argv[]);
#define GRAVITY "G"
#define P_TXT_SZ 150
#define MAX_LINKS 100
#define MAX_EQNS 1000

typedef struct {
		char name[STRLEN];
		int type;
		char *ptr;
} SETVAR;

typedef struct p {
	char txt[P_TXT_SZ];	/* text for the term       */
	struct p *next;     /* pointer to next product */
} P;

typedef struct term {
	P *p;				/* pointer to product list */
	BOOL neg;			/* leading negative sign?  */
	struct term *next;	/* pointer to next term    */
} TERM;

typedef struct child {
	struct child *next;
	struct link *link;
} CHILD;

typedef struct mon {
	float x,y;
	int num;
	struct mon *next;
} MON;

typedef struct link {
	int num;				/* number of this link */
	int parent;				/* number of parent link */
	float parx,pary;		/* point of attachment to parent */
	float mass,iner;		/* mass, moment of inertia */
	float cmx,cmy;			/* link center of gravity */
	CHILD *child;			/* pointer to children */
	struct link *next;		/* pointer to next link in linked list */
	BOOL fix;				/* is this link to be fixed? */
	BOOL mon_calc;			/* needs monitor pts calculated? */
	float pfx,pfy;			/* location of point to be fixed */
	MON *mon;				/* link list of points to be monitered */
	int linklist[MAX_LINKS];	/* how link is related to the root */
} LINK;

typedef struct eqn {
	STR label;
	TERM *terms;
	BOOL flat;
} EQN;

#define EQN_SET_SIZE 1000
typedef struct eqnset {
	int num_eqns;
	EQN eqn[EQN_SET_SIZE];
} EQNSET;
 
extern TERM *eqn_subst();
extern TERM *mul_term_eqn();
extern LINK *llookup(int);
extern char *mmalloc(int);		/* own version of malloc with error check */
extern TERM *newterm(TERM *eqn);
extern TERM *term_sub();	/* subtract one term from another */
extern TERM *term_add();	/* add two terms to each other */
extern TERM *eqncpy();		/* makes a copy of an eqn */

void cerr(int argc, STR argv[], char *errmsg);
void parse(STR line, STR parselist, int *argc, STR argv[]);
void link_children();
void findroot();	
void stats();		
void gen_vars();	
void rcalclist(LINK *link, int linklist[]);
void calc_accel(LINK *Rlink);				/* calculate acceleration equations */
void calc_force(LINK *Rlink);				/* calculate forces between links */
void gen_t_eqn();					/* generate equations for torque */
void reduce();						/* collects constants */
void find_solve_eqns();				/* find solve equations */
void flat_solve();					/* flatten required equations */
void pr_Eqns(EQNSET *eqnset,char *str);
void gem();							/* place equations in matrix */
void prVars(); 
void prLabels();
void pr_eqn_mat();
void gen_file(FILE *file);
BOOL mon_mark(LINK *Rlink);
void gen_mon_eqns(FILE *file);
void quit(int argc, STR argv[]);
void help(int argc, STR argv[]);
void echo(int argc, STR argv[]);
void cat(int argc, STR argv[]);
void pipe_in(int argc, STR argv[]);
void test(int argc, STR argv[]);
void interp(FILE *fp);
void link_children();
void pr_children();
void paddstr( TERM *term, char *str);
void paddsin(TERM *term, char *str);
void paddcos(TERM *term, char *str);
TERM *eqncpy(TERM *from);
void new_const_eqn(STR str, int num);
void new_const_term(int sign);
void new_const_var(STR temp);
void new_ptr_eqn(STR str, int num);
void findroot();
void new_ptr_term(int sign);
void new_ptr_var(char *str);
void translate(char *str);
void init_trans();
void reduce();
int term_type(TERM *term);
void insert_term(TERM **pterm, TERM *term);
void term2const(TERM **pterm);
void mfix(TERM *term);
void buildlist(LINK *link);
void add_t_cm(LINK *link);
void add_t_parent(LINK *link);
void add_t_child(LINK *link);
void add_t_fix(LINK *link);
void add_t_ext(LINK *link);
void add_t_iw(LINK *link);
void rcalc_mon(LINK *link);
void mon_local_vars(FILE *file);
void const_calc(FILE *file);
void mon_const(LINK *link);
void preqn(char *eqn_name, TERM *eqn);
void link_defs(FILE *file);
void local_vars(FILE *file);
int find_var(STR var_name);
void type_eqn(FILE *file, TERM *eqn);
void rtheta(float x, float y, float *r, float *theta);
void calc_acm(LINK *link);				/* find center of mass acc */
void calc_afix(LINK *link);			/* find fixed pt acc */
void calc_achild(LINK *link);			/* find child acc */
void prterm(TERM *term);
void calc_forcex(LINK *link);
void calc_forcey(LINK *link);
EQN *eqn_find(STR name);
void extract_ptr(TERM *term, STR var);
void flatten(EQN *eqn);
void flat_fix(EQN *eqn, STR var);
void term_ptr_rem(EQN *eqn);
void search_eqn(STR var, EQN *eqn, EQN *new_eqn);
BOOL simple_eqn(EQN *eqn);
BOOL local_term(EQN *eqn, STR var);
void term_var_rem(EQN *eqn, STR var);
void term_var_add(EQN *eqn, STR var, STR name);
BOOL term_match(TERM *term, STR var);
void add_k_term(STR var, TERM *term, EQN *new_eqn);
void add_kk_term(STR var, TERM *term, TERM *sterm, EQN *new_eqn);
void rm_term(TERM **pterm);
void negate(EQN *eqn);
void readline(FILE *file, int *argc, STR argv[]);






























