/************************************************************
	FILE: interp.c
	-procedures necessary for interpreting shell commands
************************************************************/

#include "defs.h"
#include "globals.h"

/* all commands to be interpreted must be defined here */
void compile(int nargs, STR args[]);
void fix(int nargs, STR args[]);
void create_link(int nargs, STR args[]);
void monitor(int nargs, STR args[]);
void pipe_in(int nargs, STR args[]);
void set(int nargs, STR args[]);


static struct {
	STR	fn_name;
	PFI	fn;
	int	args;
} functions[] = {
	"cat",cat,2,
	"compile",compile,1,
	"echo",echo,-1,
	"fix",fix,4,
	"link",create_link,9,
	"monitor",monitor,5,
	"<",pipe_in,2,
	"test",test,-1,
	"help",help,-1,
	"quit",quit,1,
	"set",set,-1,
   "", 		NULL,-1
};

/************************************************************
	PROC:	lookup
	INPUT:	name to look up
	OUTPUT:	pointer to function corresponding to name
	DOES:	looks up a name in the function table
************************************************************/

int lookup(char *name)
{
	int n=0;
	unsigned int maxmatch = 0;
	int matchno = 0;
	unsigned int ch;
	char *ptr1,*ptr2;
	BOOL ambig;

	/* look for exact match */
	while (functions[n].fn) {
		if (strcmp(name,functions[n].fn_name)==0)
			return(n);
		n++;
	}

	/* look for inexact match */
	n = 0;
	ambig = FALSE;
	while (functions[n].fn) {
		ptr1 = functions[n].fn_name;
		ptr2 = name;
		for (ch=0; *ptr2; ch++,ptr1++,ptr2++) {
			if (*ptr1 != *ptr2) break;
		}
		if (ch>maxmatch) {
			maxmatch = ch;
			matchno = n;
			ambig = FALSE;
		} else if (ch==maxmatch && ch==strlen(name)) ambig = TRUE;
		n++;
	}
	if (ambig)	return(-2);
	else if (maxmatch==strlen(name)) return(matchno);
	return(-1);
}

/************************************************************
	PROC:	interp(file_ptr)
	DOES:	interprets a control file
************************************************************/

void interp(FILE *file)
{
	STR argv[MAX_ARGS];
	int argc=0;
	int func;

	Linenum = 0;
	while (!feof(file)) {
		if (IMode) printf("-->");		/* print prompt */
		readline(file,&argc,argv);		/* read line of input */
		Linenum++;						/* inc current line no. */
		if (!argc) continue;			/* if empty, nothing to do */
		func = lookup(argv[0]);			/* lookup corresponding fn */
		if (func<0) {
			if (func==-1) {
				cerr(argc,argv,"Unrecognized command");
				continue;
			}
			if (func==-2) {
				cerr(argc,argv,"Ambiguous command");
				continue;
			}
		}
		if (functions[func].args != argc && functions[func].args != -1) {
			cerr(argc,argv,"Bad number of arguments");
			continue;
		}
		(*(functions[func].fn))(argc,argv);	/* execute function */
	}
}

	
