/************************************************************
	FILE:		set.c
	CONTAINS:	commands relating to setting and printing
				values of internal vars
************************************************************/

#include "defs.h"
#include "globals.h"

void prset(SETVAR *);


#define FLOAT 1
#define STRING 2
#define FVEC_TYPE 3
#define IVEC_TYPE 4
#define INTEG 5
#define FLAG 6

static SETVAR setlist[] = {
	"debug",FLAG,(char *) &DEbug,
	"dirname",STRING,(char *) DIrname,
	"filename",STRING,(char *) FIlename,
	"help",STRING,(char *) HElpfile,
	"imode",FLAG,(char *) &IMode,
	"inverse",FLAG,(char *) &INverse,
	"gravity",FLOAT,(char *) &GRavity,
	"procname",STRING,(char *) PRocname,
	"rootmon",FLAG,(char *) &ROotmon,
	"testfloat",FLOAT,(char *) &TEstfloat,
	"teststring",STRING,(char *) TEststring,
	"testfvec",FVEC_TYPE,(char *) TEstfvec,
	"testivec",IVEC_TYPE,(char *) TEstivec,
	"testint",INTEG,(char *) &TEstint,
	"testflag",FLAG,(char *) &TEstflag,
	"",0,(char *) NULL };

/************************************************************
	PROC:	set
	DOES:	sets internal variables
************************************************************/

void set(int argc, STR argv[])
{
	char *ptr;				/* pointer to variable value */
	SETVAR *setitem;		/* variable name, type and value */
	BOOL found;				/* found a match? */
	int argc2;				/* used for vectors */
	STR argv2[MAX_ARGS];	/* used for vectors */
	int n1;					/* counter used for vecs */

	if (argc==1) {
		prset(setlist);
		return;
	}
	if (argc!=3) {
		cerr(argc,argv,"Usage: set <var> <value>\n");
		return;
	}
	found = FALSE;
	setitem = setlist;
	while (setitem->ptr) {
		if (!strcmp(setitem->name,argv[1])) {
			ptr = setitem->ptr;
			found = TRUE;
			switch (setitem->type) {
				case FLOAT:
					sscanf(argv[2],"%f",(float *) ptr);
					break;
				case STRING:
					strcpy((char *) ptr,argv[2]);
					break;
				case FLAG:
					if (argv[2][0]=='t' || argv[2][0]=='T')
						*(int *) ptr = 1;
					else
						*(int *) ptr = 0;
					break;
				case INTEG:
					sscanf(argv[2],"%d",(int *) ptr);
					break;
				case IVEC_TYPE:
					parse(argv[2],",",&argc2,argv2);
					for (n1=0; n1<argc2; n1++)
						sscanf(argv2[n1],"%d",&( ((int *) ptr)[n1]));
					break;
				case FVEC_TYPE:
					parse(argv[2],",",&argc2,argv2);
					for (n1=0; n1<argc2; n1++)
						sscanf(argv2[n1],"%f",&( ((float *) ptr)[n1]));
					break;
				default:
					cerr(argc,argv,
					"Internal Error: Bad set-type");
			}
			break;
		}
		setitem++;
	}
	if (!found) {
		cerr(argc,argv,"Unrecognized set variable");
		return;
	}
}

/************************************************************
	PROC:	prset
	DOES:	prints the current values of all set variables
************************************************************/

void prset(SETVAR *setlist)
{
	char *ptr;
	int n1;

	while(setlist->ptr) {
		ptr = setlist->ptr;
		printf("%20s:\t",setlist->name);
		switch (setlist->type) {
			case FLOAT:
				printf("%f\n",*(float *)ptr);
				break;
			case STRING:
				printf("%s\n",(char *) ptr);
				break;
			case FLAG:
				if (*(int *)ptr)
					printf("TRUE\n");
				else
					printf("FALSE\n");
				break;
			case INTEG:
				printf("%d\n",* (int *) ptr);
				break;
			case FVEC_TYPE:
				for (n1=0; n1<VECSIZE; n1++)
					printf("%f ",( (float *) ptr)[n1]);
				printf("\n");
				break;
			case IVEC_TYPE:
				for (n1=0; n1<VECSIZE; n1++)
					printf("%d ",( (int *) ptr)[n1]);
				printf("\n");
				break;
			default:
				fprintf(stderr,"Internal Error in prset()\n");
		}
		setlist++;
	}
}

